import '../_shared/shared.js'; // not required, our example styling etc.
import '../../lib/Grid/column/RatingColumn.js';
import '../../lib/Scheduler/column/ResourceInfoColumn.js';
import SchedulerWithSubtasks from './lib/SchedulerWithSubtasks.js';

let scheduler;

// Cannot toggle mode between horizontal and vertical at runtime, so need to destroy and recreate the scheduler
function createScheduler(mode) {
    const isHorizontal = mode === 'horizontal';

    scheduler?.destroy?.();

    // Create a new scheduler from the custom subclass, see SchedulerWithSubtasks
    scheduler = new SchedulerWithSubtasks({
        appendTo          : 'container',
        resourceImagePath : '../_shared/images/users/',
        startDate         : new Date(2022, 2, 28, 0),
        endDate           : new Date(2022, 4, 31, 0),
        viewPreset        : 'hourAndDay',
        rowHeight         : 90,
        barMargin         : 10,
        // Columns are only applicable in horizontal mode
        columns           : isHorizontal ? [
            { type : 'resourceInfo', text : 'Name', field : 'name', width : 130 },
        ] : [],
        project : {
            autoLoad : true,
            loadUrl  : 'data/data.json',
        },
        mode,
        tbar : [
            {
                type        : 'buttongroup',
                toggleGroup : true,
                items       : {
                    horizontal : { text : 'Horizontal mode', icon : 'b-fa-left-right', pressed : isHorizontal },
                    vertical   : { text : 'Vertical mode', icon : 'b-fa-up-down', pressed : !isHorizontal }
                },
                onToggle({ pressed, source }) {
                    if (pressed) {
                        createScheduler(source.ref);
                    }
                }
            }
        ]
    });
}

createScheduler('horizontal');
